--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    15:11:48 06/19/06
-- Design Name:    
-- Module Name:    PPG - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity PPG is
    Port ( a0 : in std_logic;
           a1 : in std_logic;
           a2 : in std_logic;
           a_sign : in std_logic;
           b0 : in std_logic;
           b1 : in std_logic;
           b2 : in std_logic;
           b_sign : in std_logic;
           ancilla0 : in std_logic;
			  ancilla1 : in std_logic;
			  ancilla2 : in std_logic;
			  ancilla3 : in std_logic;
			  ancilla4 : in std_logic;
			  ancilla5 : in std_logic;
			  ancilla6 : in std_logic;
			  ancilla7 : in std_logic;
			  ancilla8 : in std_logic;
			  ancilla9 : in std_logic;
           a0_out : out std_logic;
			  a1_out : out std_logic;
			  a2_out : out std_logic;
			  b0_out : out std_logic;
			  b1_out : out std_logic;
			  b2_out : out std_logic;
           a_sign_out : out std_logic;
           b_sign_out : out std_logic;
           a0b0 : out std_logic;
           a1b0 : out std_logic;
           a0b1 : out std_logic;
           a2b0 : out std_logic;
           a0b2 : out std_logic;
           a1b1 : out std_logic;
           a1b2 : out std_logic;
           a2b1 : out std_logic;
           a2b2 : out std_logic;
           sign : out std_logic);
end PPG;

--						______________________
--		      in -->|a0		 		a0_out|--> out	= a0
--		      in -->|a1				a1_out|--> out	= a1
--		      in -->|a2				a2_out|--> out	= a2
--		      in -->|a_sign	  a_sign_out|--> out	= a_sign
--		      in -->|b0				b0_out|--> out	= b0
--		      in -->|b1				b1_out|--> out	= b1
--				in -->|b2				b2_out|--> out	= b2
--				in -->|b_sign	  b_sign_out|--> out	= b_sign
--						|							|
--						|			PPG			|
--						|							|
--		'0'	in -->|ancilla0		  a0b0|--> out = (a0 and b0) xor ancilla0	 (a0 * b0)
--		'0'	in -->|ancilla1		  a1b0|--> out = (a1 and b0) xor ancilla1	 (a1 * b0)
--		'0'	in -->|ancilla2		  a0b1|--> out = (a0 and b1) xor ancilla2	 (a0 * b1)
--		'0'	in -->|ancilla3		  a2b0|--> out = (a2 and b0) xor ancilla3	 (a2 * b0)
--		'0'	in -->|ancilla4		  a0b2|--> out = (a0 and b2) xor ancilla4	 (a0 * b2)
--		'0'	in -->|ancilla5		  a1b1|--> out = (a1 and b1) xor ancilla5	 (a1 * b1)
--		'0'	in -->|ancilla6		  a1b2|--> out = (a1 and b2) xor ancilla6	 (a1 * b2)
--		'0'	in -->|ancilla7		  a2b1|--> out = (a2 and b1) xor ancilla7	 (a2 * b1)
--		'0'	in -->|ancilla8		  a2b2|--> out = (a2 and b2) xor ancilla8	 (a2 * b2)
--		'0'	in -->|ancilla9		  sign|--> out = (a_sign xor ancilla9) xor b_sign  (result sign)
--						|____________________|
--
--	 *ancilla bits inputs are always '0' (zero)
--

architecture Behavioral of PPG is

signal intermediate_sign: std_logic;

begin

a0_out <= a0;
a1_out <= a1;
a2_out <= a2;
a_sign_out <= a_sign;
b0_out <= b0;
b1_out <= b1;
b2_out <= b2;
b_sign_out <= b_sign;

a0b0 <= (a0 and b0) xor ancilla0;
a1b0 <= (a1 and b0) xor ancilla1;
a0b1 <= (a0 and b1) xor ancilla2;
a2b0 <= (a2 and b0) xor ancilla3;
a0b2 <= (a0 and b2) xor ancilla4;
a1b1 <= (a1 and b1) xor ancilla5;
a1b2 <= (a1 and b2) xor ancilla6;
a2b1 <= (a2 and b1) xor ancilla7;
a2b2 <= (a2 and b2) xor ancilla8;

intermediate_sign <= a_sign xor ancilla9;
sign <= intermediate_sign xor b_sign;

end Behavioral;
