--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    19:04:20 06/19/06
-- Design Name:    
-- Module Name:    TPM - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity TPM is
    Port ( mul_a0 : in std_logic;
           mul_a1 : in std_logic;
           mul_a2 : in std_logic;
			  mul_a_sign : in std_logic;
			  mul_b0 : in std_logic;
           mul_b1 : in std_logic;
           mul_b2 : in std_logic;
			  mul_b_sign : in std_logic;
           mul_ancilla0 : in std_logic;
           mul_ancilla1 : in std_logic;
           mul_ancilla2 : in std_logic;
           mul_ancilla3 : in std_logic;
           mul_ancilla4 : in std_logic;
           mul_ancilla5 : in std_logic;
           mul_ancilla6 : in std_logic;
           mul_ancilla7 : in std_logic;
           mul_ancilla8 : in std_logic;
           mul_ancilla9 : in std_logic;
           mul_ancilla10 : in std_logic;
           mul_ancilla11 : in std_logic;
           mul_ancilla12 : in std_logic;
           mul_ancilla13 : in std_logic;
           mul_ancilla14 : in std_logic;
           mul_ancilla15 : in std_logic;
           mul_a0_out : out std_logic;
           mul_a1_out : out std_logic;
           mul_a2_out : out std_logic;
           mul_a_sign_out : out std_logic;
           mul_b0_out : out std_logic;
           mul_b1_out : out std_logic;
           mul_b2_out : out std_logic;
           mul_b_sign_out : out std_logic;
           mul_result0 : out std_logic;
           mul_result1 : out std_logic;
           mul_result2 : out std_logic;
           mul_result3 : out std_logic;
           mul_result4 : out std_logic;
           mul_result5 : out std_logic;
           mul_g0 : out std_logic;
           mul_g1 : out std_logic;
           mul_g2 : out std_logic;
           mul_g3 : out std_logic;
           mul_g4 : out std_logic;
           mul_g5 : out std_logic;
           mul_g6 : out std_logic;
           mul_g7 : out std_logic;
           mul_g8 : out std_logic;
           mul_sign_out : out std_logic);
end TPM;

--									 _______________________________________________________________________________
--									|																	 _________________________________
--		 (3-bits a) 	in -->|mul_a0-mul_a2	 ------------------------->	in -->|a0-a2 		    		a0_out-a2_out|-->
--		 (a sign)		in -->|mul_a_sign		 ------------------------->	in -->|a_sign	 	    			a_sign_out|-->
--		 (3-bits b) 	in -->|mul_b0-mul_b2	 ------------------------->	in -->|b0-b2		   		b0_out-b2_out|-->
--		 (b sign)		in -->|mul_b_sign		 ------------------------->	in -->|b_sign		    			b_sign_out|-->
--									|																	|				PPG
--	(16-bits ancilla) in -->|mul_ancilla0-mul_ancilla15_								|	  (partial_products)
--									|									 \
--									|									  \-(10-bits)-> 	in -->|ancilla0-ancilla9			
--									|																	|__________________________
--									|
--									|
--									|
--									|
--									|
--									|
--									|
--									|
--									|
--									|
--									|_______________________________________________________________________________
--
--

architecture Behavioral of TPM is

component TSG
	Port ( A_input1 : in std_logic;
           B_input2 : in std_logic;
           C_ancilla : in std_logic;
           D_carryin : in std_logic;
           P_garbage : out std_logic;
           Q_garbage : out std_logic;
           R_result : out std_logic;
           S_carryout : out std_logic);
end component;

component Peres
	Port (  A_input1 : in std_logic;
           B_input2 : in std_logic;
           C_ancilla : in std_logic;
           P_garbage : out std_logic;
           Q_result : out std_logic;
           R_carryout : out std_logic);
end component;

component PPG
    Port ( a0 : in std_logic;
           a1 : in std_logic;
           a2 : in std_logic;
           a_sign : in std_logic;
           b0 : in std_logic;
           b1 : in std_logic;
           b2 : in std_logic;
           b_sign : in std_logic;
           ancilla0 : in std_logic;
			  ancilla1 : in std_logic;
			  ancilla2 : in std_logic;
			  ancilla3 : in std_logic;
			  ancilla4 : in std_logic;
			  ancilla5 : in std_logic;
			  ancilla6 : in std_logic;
			  ancilla7 : in std_logic;
			  ancilla8 : in std_logic;
			  ancilla9 : in std_logic;
           a0_out : out std_logic;
			  a1_out : out std_logic;
			  a2_out : out std_logic;
			  b0_out : out std_logic;
			  b1_out : out std_logic;
			  b2_out : out std_logic;
           a_sign_out : out std_logic;
           b_sign_out : out std_logic;
           a0b0 : out std_logic;
           a1b0 : out std_logic;
           a0b1 : out std_logic;
           a2b0 : out std_logic;
           a0b2 : out std_logic;
           a1b1 : out std_logic;
           a1b2 : out std_logic;
           a2b1 : out std_logic;
           a2b2 : out std_logic;
           sign : out std_logic);
end component;

signal ppg_a0b0,ppg_a1b0,ppg_a0b1,ppg_a2b0,ppg_a0b2,ppg_a1b1,ppg_a1b2,ppg_a2b1,ppg_a2b2,ppg_sign: std_logic; -- intermediate results from ppg
signal tsg1,tsg2: std_logic; -- intermediate results from tsg's
signal c1,c2,c3,c4,c5: std_logic;  -- carry signals 

begin

partial_products: PPG
	port map( a0 => mul_a0,
          	 a1 => mul_a1,
          	 a2 => mul_a2,
          	 a_sign => mul_a_sign,
          	 b0 => mul_b0,
          	 b1 => mul_b1,
          	 b2 => mul_b2,
          	 b_sign => mul_b_sign,
         	 ancilla0 => mul_ancilla0,
				 ancilla1 => mul_ancilla1,
				 ancilla2 => mul_ancilla2,
				 ancilla3 => mul_ancilla3,
				 ancilla4 => mul_ancilla4,
				 ancilla5 => mul_ancilla5,
				 ancilla6 => mul_ancilla6,
				 ancilla7 => mul_ancilla7,
				 ancilla8 => mul_ancilla8,
				 ancilla9 => mul_ancilla9,
         	 a0_out => mul_a0_out,
				 a1_out => mul_a1_out,
			 	 a2_out => mul_a2_out,
			 	 b0_out => mul_b0_out,
			 	 b1_out => mul_b1_out,
			 	 b2_out => mul_b2_out,
          	 a_sign_out => mul_a_sign_out,
          	 b_sign_out => mul_b_sign_out,
          	 a0b0 => ppg_a0b0,
          	 a1b0 => ppg_a1b0,
          	 a0b1 => ppg_a0b1,
          	 a2b0 => ppg_a2b0,
          	 a0b2 => ppg_a0b2,
          	 a1b1 => ppg_a1b1,
          	 a1b2 => ppg_a1b2,
          	 a2b1 => ppg_a2b1,
          	 a2b2 => ppg_a2b2,
          	 sign => ppg_sign);

mul_result0 <= ppg_a0b0;

peres_level1: Peres
	port map (  A_input1 => ppg_a1b0,
           		B_input2 => ppg_a0b1,
           		C_ancilla => mul_ancilla10,
           		P_garbage => mul_g0,
           		Q_result =>	mul_result1,
           		R_carryout => c1);

TSG1_level1: TSG
	port map (  A_input1 => ppg_a2b0,
           		B_input2 => ppg_a0b2,
           		C_ancilla => mul_ancilla11,
           		D_carryin => c1,
           		P_garbage => mul_g1,
           		Q_garbage => mul_g2,
           		R_result => tsg1,
           		S_carryout => c2);

TSG2_level1: TSG
	port map (  A_input1 => ppg_a1b2,
           		B_input2 => ppg_a2b1,
           		C_ancilla => mul_ancilla12,
           		D_carryin => c2,
           		P_garbage => mul_g3,
           		Q_garbage => mul_g4,
           		R_result => tsg2,
           		S_carryout => c3);

peres1_level2: Peres
	port map (  A_input1 => tsg1,
           		B_input2 => ppg_a1b1,
           		C_ancilla => mul_ancilla13,
           		P_garbage => mul_g5,
           		Q_result =>	mul_result2,
           		R_carryout => c4);

peres2_level2: Peres
	port map (  A_input1 => c4,
           		B_input2 => tsg2,
           		C_ancilla => mul_ancilla14,
           		P_garbage => mul_g6,
           		Q_result =>	mul_result3,
           		R_carryout => c5);

TSG_level2: TSG
	port map (  A_input1 => c3,
           		B_input2 => ppg_a2b2,
           		C_ancilla => mul_ancilla15,
           		D_carryin => c5,
           		P_garbage => mul_g7,
           		Q_garbage => mul_g8,
           		R_result => mul_result4,
           		S_carryout => mul_result5);

mul_sign_out <= ppg_sign;

end Behavioral;
