--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    10:51:16 06/19/06
-- Design Name:    
-- Module Name:    TSG - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity TSG is
    Port ( A_input1 : in std_logic;
           B_input2 : in std_logic;
           C_ancilla : in std_logic;
           D_carryin : in std_logic;
           P_garbage : out std_logic;
           Q_garbage : out std_logic;
           R_result : out std_logic;
           S_carryout : out std_logic);
end TSG;

--	TSG Block:
--
--							   |in	 (carry in)
--							   |
--				   	______V______
--	 			    	|     D     |
--	(input1)	in -->|A         P|--> out	 = A 
--				   	|           |
--	(ancilla)in -->|C   TSG   Q|--> out	 = A'C' xor B' (garbage)
--				   	|           |
--	(input2)	in -->|B         R|--> out	 = (A'C' xor B') xor D (result)
--				   	|     S     |
--				   	|___________|
--							   |
--							   |
--							   V out	 = (A'C' xor B')D xor (AB xor C) (carry out)


architecture Behavioral of TSG is

signal a_neg,b_neg,c_neg: std_logic;
signal intermediate_Q: std_logic;		-- intermediate signal for Q

begin

a_neg <= not A_input1;
b_neg <= not B_input2;
c_neg <= not C_ancilla;

P_garbage <= A_input1;									-- A_input1 passes through
intermediate_Q <= (a_neg and c_neg) xor b_neg;
Q_garbage <= intermediate_Q;						 	-- garbage output bit
R_result <= intermediate_Q xor D_carryin;			-- the result
S_carryout <= (intermediate_Q and D_carryin) xor ((A_input1 and B_input2) xor C_ancilla); -- carry out


end Behavioral;
