--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    19:14:48 06/18/06
-- Design Name:    
-- Module Name:    AddSub - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity AddSub2s is
    Port ( Ain : in STD_LOGIC_VECTOR (6 downto 0);
           SignFractialA : in STD_LOGIC;
           SignVectorA : in std_logic;
           Bin : in STD_LOGIC_VECTOR (6 downto 0);
           SignFractialB : in std_logic;
           SignVectorB : in std_logic;
           Z1, Z2, Z3, Z4 : in std_logic;
           Z5, Z6, Z7, Z8 : in std_logic;
           	  G1,G2,G3,G4,G5,G6,G7,G8 : out std_logic;
              G9,G10,G11,G12,G13,G14 : out std_logic;
              G15,G16,G17 : out std_logic;-- garbage from 2's compl A, B and half_addar xor1,2			  
			  Qout : out STD_LOGIC_VECTOR (7 downto 0);
			  SignVectorQ: out std_logic);
end AddSub2s;

architecture Behavioral of AddSub2s is

component TWOsIn				-- two's complement on the input (defined in twosin.vhd)
	port ( P_M_1 : in  STD_LOGIC;  -- Plus=0, Minus=1 (Local Sign +/-)
           P_M_2 : in  STD_LOGIC;  -- Plus=0, Minus=1 (Global Sign +/-)
           I     : in  STD_LOGIC_VECTOR (6 downto 0);
           O     : out STD_LOGIC_VECTOR (6 downto 0);
           Sign  : out STD_LOGIC;
           G     : out STD_LOGIC);--Garbage bit
end component;

component TWOsOut			  -- two's complement on the output (defined in twosout.vhd)
    port ( P_M : in  STD_LOGIC; --Plus=0,Minus=1 (Output sign for full vector)
           I   : in  STD_LOGIC_VECTOR (7 downto 0);         
           O   : out STD_LOGIC_VECTOR (7 downto 0);
           Sign: out STD_LOGIC);
end component;

component PFA						-- Peres Full Adder component (defined in PFA.vhd)
      port ( Cin  : in    std_logic; 
             A    : in    std_logic; 
             B    : in    std_logic; 
             zero : in    std_logic; 
             G1   : out   std_logic; 
             G2   : out   std_logic; 
             S    : out   std_logic; 
             Cout : out   std_logic);
end component;

signal outputA: std_logic_vector (6 downto 0);
signal outputB: std_logic_vector (6 downto 0);
signal signA,signB: std_logic;
signal garbA,garbB: std_logic;
signal S: std_logic_vector (1 to 7);
signal xor_1,xor_2: std_logic;
signal c: std_logic_vector (1 to 7); -- intermediate Carry Out from PFAs

begin

A_2scomplement: TWOsIn
	port map (I(0)   =>Ain(0),
			  I(1)   =>Ain(1),
			  I(2)   =>Ain(2),
			  I(3)   =>Ain(3),
			  I(4)   =>Ain(4),
			  I(5)   =>Ain(5),
			  I(6)   =>Ain(6),
			  P_M_1=>SignFractialA,
			  P_M_2=>SignVectorA,
			  O(0)   =>outputA(0),
			  O(1)   =>outputA(1),
			  O(2)   =>outputA(2),
			  O(3)   =>outputA(3),
			  O(4)   =>outputA(4),
			  O(5)   =>outputA(5),
			  O(6)   =>outputA(6),
			  Sign =>signA,
			  G    =>G16);

B_2scomplement: TWOsIn
	port map (I(0)   =>Bin(0),
			  I(1)   =>Bin(1),
			  I(2)   =>Bin(2),
			  I(3)   =>Bin(3),
			  I(4)   =>Bin(4),
			  I(5)   =>Bin(5),
			  I(6)   =>Bin(6),
			  P_M_1=>SignFractialB,
			  P_M_2=>SignVectorB,
			  O(0)   =>outputB(0),
			  O(1)   =>outputB(1),
			  O(2)   =>outputB(2),
			  O(3)   =>outputB(3),
			  O(4)   =>outputB(4),
			  O(5)   =>outputB(5),
			  O(6)   =>outputB(6),
			  Sign =>signB,
			  G    =>G17);

Output0_add: PFA
	port map ( Cin =>Z1,
			   A   =>outputA(0),
			   B   =>outputB(0),
			   zero=>Z2,
			   G1  =>G1,
			   G2  =>G2,
			   S   =>S(1),
			   Cout=>c(1) );

Output1_add: PFA
	port map ( Cin =>c(1),
			   A   =>outputA(1),
			   B   =>outputB(1),
			   zero=>Z3,
			   G1  =>G3,
			   G2  =>G4,
			   S   =>S(2),
               Cout=>c(2) );

Output2_add: PFA
	port map ( Cin =>c(2),
			   A   =>outputA(2),
			   B   =>outputB(2),
			   zero=>Z4,
			   G1  =>G5,
			   G2  =>G6,
			   S   =>S(3),
			   Cout=>c(3) );

Output3_add: PFA
	port map ( Cin =>c(3),
			   A   =>outputA(3),
			   B   =>outputB(3),
			   zero=>Z5,
			   G1  =>G7,
			   G2  =>G8,
			   S   =>S(4),
			   Cout=>c(4) );

Output4_add: PFA
	port map ( Cin =>c(4),
			   A   =>outputA(4),
			   B   =>outputB(4),
			   zero=>Z6,
			   G1  =>G9,
			   G2  =>G10,
			   S   =>S(5),
			   Cout=>c(5) );

Output5_add: PFA
	port map ( Cin =>c(5),
			   A   =>outputA(5),
			   B   =>outputB(5),
			   zero=>Z7,
			   G1  =>G11,
			   G2  =>G12,
			   S   =>S(6),
			   Cout=>c(6) );
               
Output6_add: PFA
	port map ( Cin =>c(6),
			   A   =>outputA(6),
			   B   =>outputB(6),
			   zero=>Z8,
			   G1  =>G13,
			   G2  =>G14,
			   S   =>S(7),
			   Cout=>c(7) );
  				  
xor_1 <= signA xor signB;
xor_2 <= xor_1 xor c(7);
G15   <= signB;
 
Out_2scomplement: TWOsOUT
	port map ( I(0) => S(1),
			   I(1) => S(2),
			   I(2) => S(3),
			   I(3) => S(4),
			   I(4) => S(5),
			   I(5) => S(6),
			   I(6) => S(7),               
			   I(7) => c(7),
			   P_M  => xor_2,
			   O(0) =>Qout(0),
			   O(1) =>Qout(1),
			   O(2) =>Qout(2),
			   O(3) =>Qout(3),
			   O(4) =>Qout(4),
			   O(5) =>Qout(5),
			   O(6) =>Qout(6),
			   O(7) =>Qout(7),               
			   Sign=>SignVectorQ);				  	

end Behavioral;
